import java.io.*;
import java.util.*;

class WriteHTMLFile {
//
// WriteHTMLFile - A set of methods to create and
// write to an HTML file, encapsulating HTML-specific
// codes to a single class.
//
// David Moisan, April 1st, 1999
//
// Version 1.2
//
// License terms:  This is open source freely distributable
// under the terms of the Gnu Public License (http://www.gnu.org)
//
// Changes in this version:
//
// Comment method now fixed to produce correct SGML comments
// Doctype method added for easier validation
// Right now, the DOCTYPE is hard-coded into the constructor
// so that WriteHTMLFile uses HTML 4.0 Transitional.
// This could change
// 	
//
// These constants are added for readability and are used to 
// define whether or not a new line is written in the HTML
// output stream.

static final boolean NEWLINE = true;
static final boolean NONEWLINE = false;

   PrintStream HTMLOut;
	String HTMLDocType=" -//W3C//DTD HTML 4.0 Transitional//EN";

//
// Constructor -- Creates and opens file for output;
// 
// 


   WriteHTMLFile(String filename) throws IOException
	{
		FileOutputStream FileOut = new FileOutputStream(filename);
		this.HTMLOut = new PrintStream(FileOut);
		this.Doctype(HTMLDocType);
		this.WriteTag("<HTML>", NEWLINE);
      }    

// 
// Low level HTML output methods
//

   void WriteNewline(boolean Newline) throws IOException {
      if (Newline) {
         HTMLOut.println();
         }
      }

   void CloseHTML() throws IOException {
      this.HTMLEnd();
      HTMLOut.close();
      }

   void WriteContent(String Content) throws IOException {
		final String STAG = "<"; // Start of HTML tag
		final String ETAG = ">"; // End of HTML tag
		final String ENT = "&";  // HTML entity designator
		final String HTMLDELIM = STAG+" "+ETAG+" "+ENT;

		String NextHTMLToken;
 		StringTokenizer HTMLContentString = new StringTokenizer (Content, HTMLDELIM, true);     

		while (HTMLContentString.hasMoreTokens()) {
			NextHTMLToken = HTMLContentString.nextToken();
				
			if (NextHTMLToken.equals(STAG)) {
					HTMLOut.print("&lt;");
					}
			else if (NextHTMLToken.equals(ETAG)) {
					HTMLOut.print("&gt;");
					}
			else if (NextHTMLToken.equals(ENT)) {
					HTMLOut.print("&amp;");
					}
			else {
				HTMLOut.print(NextHTMLToken);
				}
			}
		}

//
// WriteTag -- Main method for writing tags and content
//
// There are three different method calls for convienience
//
// (Tag, Newline)
//		For simple empty tag insertion (e.g., <HTML>)
//
// (Tag, Content, Newline)
//
// 	For tags with content (such as <LI>List item)
//		NOTE:  HTML constructs like this will likely be deprecated
//		in XHTML--use the following method call instead.
//
// (Tag1, Content, Tag2, Newline)
//
//		For tags with content and closing tags such as
//		<EM>Some Emphasis</EM>
//		
// 
   void WriteTag(String Tag, boolean Newline) throws IOException {
      HTMLOut.print(Tag);
      this.WriteNewline(Newline);
      }

   void WriteTag(String Tag, String Content, boolean Newline) throws IOException {
      this.WriteTag(Tag, NONEWLINE);
      this.WriteContent(Content);
      this.WriteNewline(Newline);
      }

   void WriteTag(String Tag1, String Content, String Tag2, boolean Newline) throws IOException {
	   this.WriteTag(Tag1, NONEWLINE);
      this.WriteContent(Content);
      this.WriteTag(Tag2, Newline);
      }

//
// High level HTML methods
//
// Most of these are self-explanatory and correspond to 
// particular HTML elements
//
//
	void Doctype(String Content) throws IOException {
		this.WriteTag("<!DOCTYPE HTML PUBLIC \"",Content,"\">", NEWLINE);
		}

	void HTMLStart() throws IOException {
		this.WriteTag("<HTML>", NEWLINE);
		}

	void HTMLEnd() throws IOException {
		this.WriteTag("</HTML>", NEWLINE);
		}

	void Comment(String Content) throws IOException {
		this.WriteTag("<!-- ",Content," -->", NEWLINE);
		}

	void TitleTag(String Content) throws IOException {
		this.WriteTag("<TITLE>",Content,"</TITLE>", NEWLINE);
		}
	
	void BodyTag() throws IOException {
		this.WriteTag("<BODY>", NEWLINE);
		}

	void BodyEndTag() throws IOException {
		this.WriteTag("</BODY>", NEWLINE);		
		}

	void HeadTag() throws IOException {	
		this.WriteTag("<HEAD>", NEWLINE);
		}

	void HeadEndTag() throws IOException {
		this.WriteTag("</HEAD>", NEWLINE);
		}

	void ParaTag() throws IOException {
		this.WriteNewline(NEWLINE);
		this.WriteTag("<P>", NONEWLINE);
		}

	void ParaEndTag() throws IOException {
		this.WriteTag("</P>", NEWLINE);
		}

	void EmphTag() throws IOException {
		this.WriteTag("<EM>", NONEWLINE);
		}

	void EmphTag(String Content) throws IOException {
		this.WriteTag("<EM>", Content, "</EM>", NONEWLINE);
		}

	void EmphEndTag() throws IOException {
		this.WriteTag("</EM>", NONEWLINE);
		}

	void CiteTag() throws IOException {
		this.WriteTag("<CITE>", NONEWLINE);
		}

	void CiteTag(String Content) throws IOException {
		this.WriteTag("<CITE>", Content, "</CITE>", NONEWLINE);
		}

	void CiteEndTag() throws IOException {
		this.WriteTag("</CITE>", NONEWLINE);
		}

	void HeadingTag(int Level, String Content) throws IOException {
		this.WriteTag("<H"+Level+">", Content, "</H"+Level+">", NEWLINE);
		}

	void HorizRuleTag() throws IOException {
		this.WriteTag("<HR>", NEWLINE);
		}

	void LineBreakTag() throws IOException {
		this.WriteNewline(NEWLINE);
		this.WriteTag("<BR>", NONEWLINE);
		}

	void UnordListTag() throws IOException {
		this.WriteNewline(NEWLINE);
		this.WriteTag("<UL>", NEWLINE);
		}

	void UnordListEndTag() throws IOException {
		this.WriteNewline(NEWLINE);
		this.WriteTag("</UL>", NEWLINE);
		}

	void OrdListTag() throws IOException {
		this.WriteNewline(NEWLINE);
		this.WriteTag("<OL>", NEWLINE);
		}

	void OrdListEndTag() throws IOException {
		this.WriteNewline(NEWLINE);
		this.WriteTag("</OL>", NEWLINE);
		}
   
	void ListItemTag(String Content) throws IOException {
		this.WriteTag("<LI>",Content,"</LI>", NEWLINE);
		}
	
	void ListItemTag() throws IOException {
		this.WriteTag("<LI>", NONEWLINE);
		}
	
	void ListItemEndTag() throws IOException {
		this.WriteTag("</LI>", NONEWLINE);
		}

//
// Link tag methods
//
// Note that the URL (HREF and NAME) is _not_ in quotes!
//

	void AnchorLink(String URL, String Content, boolean newline) throws IOException {
		this.WriteTag("<A HREF=\""+URL+"\">", Content,"</A>", newline);
		}

	void NameLink(String Name, String Content, boolean newline) throws IOException {
		this.WriteTag("<A NAME=\""+Name+"\">", Content,"</A>",newline);
		}
}
 